package com.ruoyi.project.reliability.mapper;

import java.util.List;
import java.util.Map;

import com.ruoyi.framework.aspectj.lang.annotation.DataScope;
import com.ruoyi.project.common.domain.DataEntity;
import com.ruoyi.project.reliability.domain.TBamOriginal;

/**
 * 故障管理初选Mapper接口
 *
 * @author ruoyi
 * @date 2020-11-27
 */
public interface TBamOriginalMapper
{
    /**
     * 查询故障管理初选
     *
     * @param id 故障管理初选ID
     * @return 故障管理初选
     */
    public TBamOriginal selectTBamOriginalById(Long id);

    /**
     * 查询故障管理初选列表
     *
     * @param tBamOriginal 故障管理初选
     * @return 故障管理初选集合
     */
    @DataScope(deptAlias = "d")
    public List<TBamOriginal> selectTBamOriginalList(TBamOriginal tBamOriginal);

    /**
     * 新增故障管理初选
     *
     * @param tBamOriginal 故障管理初选
     * @return 结果
     */
    public int insertTBamOriginal(TBamOriginal tBamOriginal);

    /**
     * 修改故障管理初选
     *
     * @param tBamOriginal 故障管理初选
     * @return 结果
     */
    public int updateTBamOriginal(TBamOriginal tBamOriginal);

    /**
     * 删除故障管理初选
     *
     * @param id 故障管理初选ID
     * @return 结果
     */
    public int deleteTBamOriginalById(Long id);

    /**
     * 批量删除故障管理初选
     *
     * @param ids 需要删除的数据ID
     * @return 结果
     */
    public int deleteTBamOriginalByIds(Long[] ids);

    List<DataEntity> selectPriorityData(Map param);

    List<DataEntity> selectDisciplineData(Map param);
}
